/*
 * Error handling routines.
 *
 * The functions in this file are independent of any application
 * variables, and may be used with any C program.
 * Either of the names CLIENT or SERVER may be defined when compiling
 * this function.  If neither are defined, we assume CLIENT.
 */

#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include	<stdio.h>
#include	<stdarg.h>
#include <errno.h>

#include	"systype.h"

#ifdef	CLIENT
#ifdef	SERVER
cant define both CLIENT and SERVER
#endif
#endif

#ifndef	CLIENT
#ifndef	SERVER
#define	CLIENT	1		/* default to client */
#endif
#endif

#ifndef	NULL
#define	NULL	((void *) 0)
#endif

/*char	*pname = NULL;*/

#ifdef	CLIENT			/* these all output to stderr */

/*
 * Fatal error.  Print a message and terminate.
 * Don't dump core and don't print the system's errno value.
 *
 *	err_quit(str, arg1, arg2, ...)
 *
 * The string "str" must specify the conversion specification for any ap.
 */

/*VARARGS1*/
err_quit(char *fmt, ...)
{
	va_list		ap;

	va_start(ap,fmt);
	ap = va_arg(ap, char *);
	if (ap != NULL)
	         vfprintf(stderr, fmt, ap);
        else
		fprintf(stderr, fmt);
	fputc('\n', stderr);
	va_end(ap);

	exit(1);
}

/*
 * Fatal error related to a system call.  Print a message and terminate.
 * Don't dump core, but do print the system's errno value and its
 * associated message.
 *
 *	err_sys(str, arg1, arg2, ...)
 *
 * The string "str" must specify the conversion specification for any ap.
 */

/*VARARGS1*/
err_sys(char *fmt, ...)
{
	va_list		ap;

	va_start(ap,fmt);
	ap = va_arg(ap, char *);
	if (ap != NULL)
		fprintf(stderr, fmt , ap);
        else 
            fprintf(stderr, fmt );
	va_end(ap);

	my_perror();

	exit(1);
}

/*
 * Recoverable error.  Print a message, and return to caller.
 *
 *	err_ret(str, arg1, arg2, ...)
 *
 * The string "str" must specify the conversion specification for any ap.
 */

/*VARARGS1*/
err_ret(char *fmt, ...)
{
	va_list		ap;

	va_start(ap,fmt);
	ap = va_arg(ap, char *);
        if (ap != NULL )
	     fprintf(stderr, fmt, ap);
        else
             fprintf(stderr, fmt);
	va_end(ap);

	my_perror();

	fflush(stdout);
	fflush(stderr);

	return;
}

/*
 * Fatal error.  Print a message, dump core (for debugging) and terminate.
 *
 *	err_dump(str, arg1, arg2, ...)
 *
 * The string "str" must specify the conversion specification for any ap.
 */

/*VARARGS1*/
err_dump(char *fmt, ...)
{
	va_list		ap;

	va_start(ap,fmt);
	ap = va_arg(ap, char *);
	if (ap != NULL)
	        fprintf(stderr, fmt, ap);
        else
		fprintf(stderr, fmt);
	va_end(ap);

	my_perror();

	fflush(stdout);		/* abort doesn't flush stdio buffers */
	fflush(stderr);

	abort();		/* dump core and terminate */
	exit(1);		/* shouldn't get here */
}

/*
 * Print the UNIX errno value.
 */

my_perror()
{
	char	*sys_err_str();

	fprintf(stderr, " %s\n", sys_err_str());
}

#endif	/* CLIENT */

#ifdef	SERVER

#ifdef	BSD
/*
 * Under BSD, these server routines use the syslog(3) facility.
 * They don't append a newline, for example.
 */

#include	<syslog.h>

#else	/* not BSD */
/*
 * There really ought to be a better way to handle server logging
 * under System V.
 */

#define	syslog(a,b)	fprintf(stderr, "%s\n", (b))
#define	openlog(a,b,c)	fprintf(stderr, "%s\n", (a))

#endif	/* BSD */

char	emesgstr[255] = {0};	/* used by all server routines */

/*
 * Identify ourself, for syslog() messages.
 *
 * LOG_PID is an option that says prepend each message with our pid.
 * LOG_CONS is an option that says write to console if unable to send
 * the message to syslogd.
 * LOG_DAEMON is our facility.
 */

err_init(ident)
char	*ident;
{
	openlog(ident, (LOG_PID | LOG_CONS), LOG_DAEMON);
}

/*
 * Fatal error.  Print a message and terminate.
 * Don't print the system's errno value.
 *
 *	err_quit(str, arg1, arg2, ...)
 *
 * The string "str" must specify the conversion specification for any ap.
 */

/*VARARGS1*/
err_quit(char *fmt,...)
{
	va_list		ap;

	va_start(ap,fmt);
	ap = va_arg(ap, char *);
	vsprintf(emesgstr, fmt, ap);
	va_end(ap);

	syslog(LOG_ERR, emesgstr);

	exit(1);
}

/*
 * Fatal error related to a system call.  Print a message and terminate.
 * Don't dump core, but do print the system's errno value and its
 * associated message.
 *
 *	err_sys(str, arg1, arg2, ...)
 *
 * The string "str" must specify the conversion specification for any ap.
 */

/*VARARGS1*/
err_sys(char *fmt, ...)
{
	va_list		ap;

	va_start(ap,fmt);
	ap = va_arg(ap, char *);
	vsprintf(emesgstr, fmt, ap);
	va_end(ap);

	my_perror();
	syslog(LOG_ERR, emesgstr);

	exit(1);
}

/*
 * Recoverable error.  Print a message, and return to caller.
 *
 *	err_ret(str, arg1, arg2, ...)
 *
 * The string "str" must specify the conversion specification for any ap.
 */

/*VARARGS1*/
err_ret(char *fmt, ...)
{
	va_list		ap;

	va_start(ap,fmt);
	ap = va_arg(ap, char *);
	vsprintf(emesgstr, fmt, ap);
	va_end(ap);

	my_perror();
	syslog(LOG_ERR, emesgstr);

	return;
}

/*
 * Fatal error.  Print a message, dump core (for debugging) and terminate.
 *
 *	err_dump(str, arg1, arg2, ...)
 *
 * The string "str" must specify the conversion specification for any ap.
 */

/*VARARGS1*/
err_dump(char *fmt, ...)
{
	va_list		ap;

	va_start(ap,fmt);
	ap = va_arg(ap, char *);
	vsprintf(emesgstr, fmt, ap);
	va_end(ap);

	my_perror();
	syslog(LOG_ERR, emesgstr);

	abort();		/* dump core and terminate */
	exit(1);		/* shouldn't get here */
}

/*
 * Print the UNIX errno value.
 * We just append it to the end of the emesgstr[] array.
 */

my_perror()
{
	register int	len;
	char		*sys_err_str();

	len = strlen(emesgstr);
	sprintf(emesgstr + len, " %s", sys_err_str());
}

#endif	/* SERVER */

			/* remainder is for both CLIENT and SERVER */
extern int	errno;		/* Unix error number */

#ifdef	SYS5
int	t_errno;	/* in case caller is using TLI, these are "tentative
			   definitions"; else they're "definitions" */
int	t_nerr;
#ifdef SYS5_4
extern char *t_errlist[];
#else
char	*t_errlist[1];
#endif
#endif


/*
 * Return a string containing some additional operating-system
 * dependent information.
 * Note that different versions of UNIX assign different meanings
 * to the same value of "errno" (compare errno's starting with 35
 * between System V and BSD, for example).  This means that if an error
 * condition is being sent to another UNIX system, we must interpret
 * the errno value on the system that generated the error, and not
 * just send the decimal value of errno to the other system.
 */

char *
sys_err_str()
{
	static char	msgstr[200];

	if (errno != 0) {
			sprintf(msgstr, "(%s)", strerror(errno));
	} else {
		msgstr[0] = '\0';
	}

#ifdef	SYS5
	if (t_errno != 0) {
		char	tmsgstr[100];

			sprintf(tmsgstr, " (%s)", strerror(t_errno));


		strcat(msgstr, tmsgstr);	/* catenate strings */
	}
#endif

	return(msgstr);
}
